## DSE driver configuration

The DSE driver uses the [same configuration mechanism][oss] as the OSS driver.

It bundles its own `dse-reference.conf` file for configuration defaults, that inherits from the OSS
driver's `reference.conf`. This allows us not to repeat everything in the DSE configuration file, it
only contains DSE-specific options.

For example, the OSS driver's `reference.conf` defines a default request consistency level:

```
# reference.conf in com.datastax.oss::java-driver-core
datastax-java-driver {
  basic.request.consistency = LOCAL_ONE
}
```

The DSE driver's `dse-reference.conf` does not override that option, so it inherits `LOCAL_ONE`. On
the other hand, it defines its own defaults for DSE-specific features, for example the traversal
source for graph statements: 

```
# dse-reference.conf in com.datastax.dse::dse-java-driver-core
datastax-java-driver {
  basic.graph.traversal-source = g
}
```

So if you execute a graph statement with the DSE driver, you get a default consistency of
`LOCAL_ONE` and the default traversal source `g`. If you put an `application.conf` in your
application classpath, you can override either of those defaults:

```
# application.conf in your application
datastax-java-driver {
  basic {
    request.consistency = ONE
    graph.traversal-source = a
  }
}
```

### Advanced topics

If you are providing your own `DriverConfigLoader` implementation (for example to [change the config
prefix] or [load from a different source]), here's how you can inherit the driver's built-in
defaults:

```java
DriverConfigLoader myConfigLoader =
    new DefaultDriverConfigLoader(
        () -> {
          // Load from your custom source / prefix
          Config myConfig = ...;
          
          // Load dse-reference.conf from the DSE driver artifact
          Config dseConfig =
              ConfigFactory.parseResourcesAnySyntax("dse-reference")
                  .getConfig("datastax-java-driver");
          
          // Load reference.conf from the OSS driver artifact
          Config ossConfig =
              ConfigFactory.parseResourcesAnySyntax("reference")
                  .getConfig("datastax-java-driver");
          
          return myConfig.withFallback(dseConfig).withFallback(ossConfig);
        });

DseSession session = DseSession.builder().withConfigLoader(myConfigLoader).build();
```

[oss]: https://docs.datastax.com/en/developer/java-driver/4.0/manual/core/configuration/
[change the config prefix]: https://docs.datastax.com/en/developer/java-driver/4.0/manual/core/configuration/#changing-the-config-prefix
[load from a different source]: https://docs.datastax.com/en/developer/java-driver/4.0/manual/core/configuration/#loading-from-a-different-source
